@@ -4,7 +4,7 @@ Copyright (C) 2010-2018 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
+the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -20,21 +20,6 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_ADD: autoload ("network_get_info", "parallel_interface.oct");
 // PKG_DEL: autoload ("network_get_info", "parallel_interface.oct", "remove");
 
-#include <octave/oct.h>
-#include <octave/dirfns.h>
-#include <octave/oct-map.h>
-#include <octave/oct-env.h>
-
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/poll.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <errno.h>
-#include <netdb.h>
-#include <unistd.h>
-
 #include "parallel-gnutls.h"
 
 DEFUN_DLD (network_get_info, args, ,
