@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2015-2018 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015-2019 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -28,18 +28,11 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 #endif
 #endif
 
-#include <sys/socket.h>
-
-#include <netinet/in.h>
-
-#include <netdb.h>
-
-#include <arpa/inet.h>
-
 #include <string.h>
 
-#include <poll.h>
+#include <signal.h>
 
+#include "parallel-gnutls.h"
 
 enum encryption_type
   {
@@ -49,6 +42,139 @@ enum encryption_type
 
 #define PROTOCOL_VERSION 1
 
+extern volatile sig_atomic_t octave_parallel_interrupted;
+
+bool octave_parallel_assert_maybe_inthandler (void);
+
+void octave_parallel_inthandler (int signal);
+
+class
+octave_parallel_maybe_inthandler
+{
+  // This class is meant to have only one object instantiated.
+
+public:
+
+  octave_parallel_maybe_inthandler (void)
+    : store_errno (0), oldsigset (NULL)
+  {
+    oldsigset = gnulib_alloc_sigset ();;
+
+    if (gnulib_get_sigmask (oldsigset))
+      {
+        store_errno = errno;
+        return;
+      }
+
+    if (gnulib_sigismember (oldsigset, SIGINT))
+      {
+        // SIGINT is blocked. (We probably have a (newer) version of
+        // Octave in which there is a separate signal handling
+        // thread.) Install a signal handler; it can remain installed,
+        // it will only have an effect during the periods when we
+        // temporarily unblock SIGINT to make some system calls
+        // interruptable.
+
+        if (gnulib_install_sighandler (SIGINT, &octave_parallel_inthandler))
+          {
+            store_errno = errno;
+
+            return;
+          }
+      }
+  }
+
+  ~octave_parallel_maybe_inthandler (void)
+  {
+    free (oldsigset);
+  }
+
+  bool good (void)
+  {
+    return store_errno == 0;
+  }
+
+private:
+
+  int store_errno;
+
+  void *oldsigset;
+};
+
+// temporarily remove SIGINT from set of blocked signals; there are
+// still racing conditions which may necessitate hitting ctrl-c more
+// than once
+class
+octave_parallel_open_sigmask
+{
+public:
+
+  octave_parallel_open_sigmask (void)
+    : restore_sigmask (false), oldsigset (NULL)
+  {
+    oldsigset = gnulib_alloc_sigset ();
+
+    if (gnulib_get_sigmask (oldsigset)
+        || gnulib_unblock (initialize_newset ()))
+      return;
+
+    restore_sigmask = true;
+  }
+
+  ~octave_parallel_open_sigmask (void)
+  {
+    if (restore_sigmask)
+      gnulib_set_sigmask (oldsigset);
+
+    free (oldsigset);
+  }
+
+private:
+
+  bool restore_sigmask;
+
+  void *oldsigset;
+
+  class
+  sigset_guard
+  {
+  public:
+
+    sigset_guard (void) : newsigset (NULL)
+    {
+      newsigset = gnulib_alloc_sigset ();
+    }
+
+    void *get (void)
+    {
+      return newsigset;
+    }
+
+    ~sigset_guard (void)
+    {
+      free (newsigset);
+    }
+
+  private:
+
+    void *newsigset;
+  };
+
+  static void *initialize_newset (void)
+  {
+    static bool first_call = true;
+    static sigset_guard __sigset_guard;
+
+    if (first_call)
+      {
+        gnulib_set_newset_one_signal (__sigset_guard.get (), SIGINT);
+
+        first_call = false;
+      }
+
+    return __sigset_guard.get ();
+  }
+};
 
 /*
 Although we use the standard file format, we can't rely on
@@ -87,10 +213,14 @@ octave_parallel_streambuf : public std::streambuf
 public:
 
   octave_parallel_streambuf (void)
-    : readable (false), writeable (false), fid (-1), store_errno (0)
+    : readable (false), writeable (false), fid (-1), store_errno (0),
+      terminated (false)
   {
     // no character in the one-character-wide buffer at the beginning
     setg (&inbuf, (&inbuf) + 1, (&inbuf) + 1);
+
+    if (! octave_parallel_assert_maybe_inthandler ())
+      store_errno = -1;
   }
 
   virtual ~octave_parallel_streambuf (void)
@@ -98,16 +228,26 @@ public:
     inbuf = '\0';
   }
 
-  virtual bool good (void) = 0;
-
-  virtual bool connection_state_undefined (void)
+  bool good (void)
   {
-    return false;
+    return (store_errno == 0 && ! terminated);
   }
 
-  virtual void set_connection_state_undefined (void)
+  bool check_interrupt_handler (void)
   {
+    if (octave_parallel_interrupted)
+      {
+        terminated = true;
 
+        return true;
+      }
+    else
+      return false;
+  }
+
+  bool io_error (void)
+  {
+    return (store_errno);
   }
 
   void close (void)
@@ -135,26 +275,29 @@ public:
       return -1;
 
     if (count == 0)
-      return 0; // if nothing needs to be written, there's no need to
-                // risk write() still being interrupted by a signal
+      return 0;
 
     ssize_t ret;
 
     size_t left = count;
 
-    // If write returns zero though count is positive, the reason must
-    // be something different than being interrupted by a signal,
-    // since in the latter case write would return -1 if no data was
-    // written at all.
-    do
-      {
-        if ((ret = do_write ((char *) buf + count - left, left)) > 0)
-          left -= ret;
-      }
-    while (left && ret > 0); // don't repeat interrupted write()
-    // while (left &&
-    //        ((ret == -1 && errno == EINTR) ||
-    //         ret > 0));
+    {
+      octave_parallel_interrupted = 0;
+
+      octave_parallel_open_sigmask dummy;
+
+      call_octave_quit ();
+
+      do
+        {
+          if ((ret = do_write ((char *) buf + count - left, left)) > 0)
+            left -= ret;
+
+          call_octave_quit ();
+        }
+      while (left && ret > 0 && ! check_interrupt_handler ());
+
+    }
 
     if (ret == -1)
       store_errno = errno;
@@ -279,7 +422,7 @@ public:
 
   int network_send_4byteint (uint32_t v, bool)
   {
-    uint32_t t = htonl (v);
+    uint32_t t = gnulib_htonl (v);
 
     write ((void *) &t, 4);
 
@@ -298,7 +441,7 @@ public:
     if (! good ())
       return -1;
 
-    v = ntohl (t);
+    v = gnulib_ntohl (t);
 
     return 0;
   }
@@ -339,7 +482,7 @@ public:
     if (! good ())
       return -1;
 
-    cast.u = ntohl (t);
+    cast.u = gnulib_ntohl (t);
 
     v = cast.s;
 
@@ -357,7 +500,7 @@ public:
   {
     uint32_t l = strlen (s);
 
-    uint32_t t = htonl (l);
+    uint32_t t = gnulib_htonl (l);
 
     write ((void *) &t, 4);
 
@@ -378,7 +521,7 @@ public:
     if (! good ())
       return -1;
 
-    l = ntohl (t);
+    l = gnulib_ntohl (t);
 
     if (l > maxl - 1)
       {
@@ -406,7 +549,7 @@ public:
     if (! good ())
       return -1;
 
-    l = ntohl (t);
+    l = gnulib_ntohl (t);
 
     struct __bufguard
     {
@@ -460,6 +603,15 @@ protected:
 
   int store_errno;
 
+  void call_octave_quit (void)
+  {
+    terminated = true;
+
+    OCTAVE_QUIT;
+
+    terminated = false;
+  }
+
   virtual ssize_t do_write (const void *buf, size_t count) = 0;
 
   virtual ssize_t do_read (void *buf, size_t count) = 0;
@@ -484,19 +636,23 @@ private:
 
     size_t left = count;
 
-    // If read returns zero though count is positive, the reason must
-    // be something different than being interrupted by a signal,
-    // since in the latter case read would return -1 if no data was
-    // written at all.
-    do
-      {
-        if ((ret = do_read ((char *) buf + count - left, left)) > 0)
-          left -= ret;
-      }
-    while (left && ret > 0); // don't repeat interrupted read()
-    // while (left &&
-    //        ((ret == -1 && errno == EINTR) ||
-    //         ret > 0));
+    {
+      octave_parallel_interrupted = 0;
+
+      octave_parallel_open_sigmask dummy;
+
+      call_octave_quit ();
+
+      do
+        {
+          if ((ret = do_read ((char *) buf + count - left, left)) > 0)
+            left -= ret;
+
+          call_octave_quit ();
+        }
+      while (left && ret > 0 && ! check_interrupt_handler ());
+
+    }
 
     if (ret == -1)
       store_errno = errno;
@@ -526,8 +682,55 @@ private:
         first_time = false;
       }
   }
+
+  bool terminated;
 }; // class octave_parallel_streambuf
 
+class
+octave_parallel_pipe_streambuf : public octave_parallel_streambuf
+{
+public:
+
+  octave_parallel_pipe_streambuf (octave_parallel::pipes::pipe_wrapper &pw_arg,
+                                  bool in)
+    : pw (pw_arg)
+  {
+    if (in)
+      readable = true;
+    else
+      writeable = true;
+
+    // this is a dummy
+    fid = 1;
+  }
+
+  ~octave_parallel_pipe_streambuf (void)
+  {
+
+  }
+
+protected:
+
+  ssize_t do_read (void *buf, size_t count)
+  {
+    return pw.read (buf, count);
+  }
+
+  ssize_t do_write (const void *buf, size_t count)
+  {
+    return pw.write (buf, count);
+  }
+
+  int do_close (void)
+  {
+    return pw.close ();
+  }
+
+private:
+
+  octave_parallel::pipes::pipe_wrapper &pw;
+};
+
 // To avoid user space buffers, provided by external libraries, which
 // can not be zero'ed after use, we have to directly use system calls
 // for reading and writing sensitive data from/to files. Luckily, for
@@ -566,11 +769,6 @@ public:
       while (::close (fid) == -1 && errno == EINTR);
   }
 
-  bool good (void)
-  {
-    return (store_errno == 0);
-  }
-
 protected:
 
   ssize_t do_read (void *buf, size_t count)
@@ -589,12 +787,6 @@ protected:
   }
 }; // class octave_parallel_file_streambuf
 
-#define BEGIN_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE_PSSTREAMBUF \
-  BEGIN_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE_1; \
-  undefined_connection_state = true; \
-  octave_rethrow_exception (); \
-  BEGIN_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE_2
-
 // TCP sockets without authentication or encryption, as an alternative
 // to encrypted connections (currently only TLS-SRP), which are (is)
 // derived from this.
@@ -619,65 +811,29 @@ public:
   ~octave_parallel_socket_streambuf (void)
   {
     if (fid >= 0)
-      while (::close (fid) == -1 && errno == EINTR);
-  }
-
-  bool good (void)
-  {
-    return (store_errno == 0);
-  }
-
-  bool connection_state_undefined (void)
-  {
-    return undefined_connection_state;
-  }
-
-  void set_connection_state_undefined (void)
-  {
-    undefined_connection_state = true;
+      while (gnulib_close (fid) == -1 && errno == EINTR);
   }
 
   // redefined streambuf virtual functions
 
-  std::streamsize showmanyc ()
-  {
-    if (! good () || fid == -1)
-      return -1;
-
-    struct pollfd pfd;
-    pfd.fd = fid;
-    pfd.events = POLLIN;
-    pfd.revents = 0;
-
-    std::streamsize ret;
-    if ((ret = poll (&pfd, 1, 0)) == 1 && pfd.revents & POLLIN)
-      return 1;
-    else
-      {
-        if (ret < 0)
-          store_errno = -1;
-        return ret;
-      }
-  }
+  std::streamsize showmanyc (void);
 
 protected:
 
   ssize_t do_read (void *buf, size_t count)
   {
-    return ::recv (fid, buf, count, 0);
+    return gnulib_recv (fid, buf, count, 0);
   }
 
   ssize_t do_write (const void *buf, size_t count)
   {
-    return ::send (fid, buf, count, 0);
+    return gnulib_send (fid, buf, count, 0);
   }
 
   int do_close (void)
   {
-    return ::close (fid);
+    return gnulib_close (fid);
   }
-
-  bool undefined_connection_state;
 };
 
 // for usage with class octave_parallel_gnutls_streambuf, one parent
@@ -852,11 +1008,6 @@ public:
       }
   }
 
-  bool good (void)
-  {
-    return (store_errno == 0);
-  }
-
   const char *server_get_username (void)
   {
     return gnutls_srp_server_get_username (session);
@@ -869,11 +1020,23 @@ public:
 
     int ret;
     // this is a dummy alert to force flushing
-    while ((ret = gnutls_alert_send
-            (session, GNUTLS_AL_WARNING, GNUTLS_A_INTERNAL_ERROR)) ==
-           GNUTLS_E_AGAIN); // don't repeat if interrupted
-    // GNUTLS_E_INTERRUPTED ||
-    // ret == GNUTLS_E_AGAIN);
+    {
+      octave_parallel_interrupted = 0;
+
+      octave_parallel_open_sigmask dummy;
+
+      call_octave_quit ();
+
+      do
+        {
+          ret = gnutls_alert_send
+            (session, GNUTLS_AL_WARNING, GNUTLS_A_INTERNAL_ERROR);
+
+          call_octave_quit ();
+        }
+      while (ret == GNUTLS_E_AGAIN && ! check_interrupt_handler ());
+
+    }
 
     if (ret != GNUTLS_E_SUCCESS)
       {
@@ -896,10 +1059,10 @@ protected:
 
     if (ret < 0)
       {
-        // although checking GNUTLS_E_AGAIN is probably not necessary
-        // with blocking calls configured
-        if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN)
+        if (ret == GNUTLS_E_INTERRUPTED)
           errno = EINTR;
+        else if (ret == GNUTLS_E_AGAIN)
+          errno = EAGAIN;
 
         ret = -1;
       }
@@ -917,10 +1080,10 @@ protected:
 
     if (ret < 0)
       {
-        // although checking GNUTLS_E_AGAIN is probably not necessary
-        // with blocking calls configured
-        if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN)
+        if (ret == GNUTLS_E_INTERRUPTED)
           errno = EINTR;
+        else if (ret == GNUTLS_E_AGAIN)
+          errno = EAGAIN;
 
         ret = -1;
       }
@@ -986,6 +1149,11 @@ public:
     return strbuf->good ();
   }
 
+  bool io_error (void) const
+  {
+    return strbuf->io_error ();
+  }
+
   void close (void) const
   {
     if (strbuf)
