@@ -1,10 +1,10 @@
 /*
 
-Copyright (C) 2015-2018 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015-2020 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
+the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -36,38 +36,13 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 #include <fcntl.h>
 
-#include <sys/mman.h>
-
 #include <errno.h>
 
-#include <sys/wait.h>
-
 #include <stdio.h>
 
-#ifdef HAVE_MALLOC_H
-#include <malloc.h>
-#elif HAVE_SYS_MALLOC_H
-#include <sys/malloc.h>
-#else
-#error "No malloc.h present."
-#endif
-
-/*
-not necessary with current Octave
-#include <stdlib.h>
-#include <string.h>
-*/
-
 #include <unistd.h>
-// next is from gnulib, getpass of unistd.h is deprecated
-/* Get getpass declaration, if available.  */
-// # if defined HAVE_DECL_GETPASS && !HAVE_DECL_GETPASS
-#ifndef HAVE_GETPASS
-/* Read a password of arbitrary length from /dev/tty or stdin.  */
-extern "C" {
-char *getpass (const char *prompt);
-}
-#endif
+
+#include <stdint.h>
 
 // We link against the gnulib num_processors() used by Octave. nproc.h
 // used by Octave is not accessible. If the interface changes, this
@@ -102,42 +77,20 @@ extern unsigned long int num_processors (enum nproc_query query);
 
 #define N_CONNECT_RETRIES 10
 
-/* define some of the following for debugging, but take care:
-   sensitive data (e.g. the password) will be written to standard
-   error and to logfiles */
-#undef octave_parallel_debug_server
-// #define octave_parallel_debug_server 1
-#undef octave_parallel_debug_client
-// #define octave_parallel_debug_client 1
-#undef octave_parallel_debug_lib
-// #define octave_parallel_debug_lib 1
-
-#ifdef octave_parallel_debug_server
-  #define dsprintf(...) fprintf (stderr, __VA_ARGS__); fflush (stderr);
-#else
-  #define dsprintf(...)
-#endif
-
-#ifdef octave_parallel_debug_client
-  #define dcprintf(...) fprintf (stderr, __VA_ARGS__);
-#else
-  #define dcprintf(...)
-#endif
-
-#ifdef octave_parallel_debug_lib
-  #define dlprintf(...) fprintf (stderr, __VA_ARGS__); fflush (stderr);
-#else
-  #define dlprintf(...)
-#endif
-
 // parallel package includes
 
+#include "parallel-gnutls-nonoctave.h"
+
 #include "error-helpers.h"
 
 #ifdef HAVE_LIBGNUTLS
 #include "sensitive-data.h"
 #endif
 
+#include "gnulib-wrappers.h"
+
+#include "p-pipes.h"
+
 #include "p-streams.h"
 
 #include "p-connection.h"
@@ -146,6 +99,8 @@ extern unsigned long int num_processors (enum nproc_query query);
 
 #include "minimal-load-save.h"
 
+#include "p-control.h"
+
 #ifdef HAVE_LIBGNUTLS
 void parallel_gnutls_set_mem_functions (void);
 #endif
