@@ -21,9 +21,10 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 static int assert_dir (std::string &path)
 {
-  OCTAVE__SYS__FILE_STAT stat (path);
+  // Windows headers expand stat(), so stat_ is used as object name
+  OCTAVE__SYS__FILE_STAT stat_ (path);
 
-  if (! stat.is_dir () && mkdir (path.c_str (), 0700))
+  if (! stat_.is_dir () && gnulib_mkdir_mode_0700 (path.c_str ()))
     return -1;
   else
     return 0;
@@ -48,8 +49,8 @@ passing as the argument @var{options} a structure with a value of\n\
 \n\
 For authentication, subdir @code{server}, and possibly subdir\n\
 @code{client}, have to be placed together with their contents at the\n\
-respective machines. They can either be placed under the directory\n\
-given by:\n\
+respective machines (but see \"Notes\" below). They can either be\n\
+placed under the directory given by:\n\
 \n\
 @code{fullfile (OCTAVE_HOME (), \"share\", \"octave\", \"parallel-srp-data\")}\n\
 \n\
@@ -61,6 +62,15 @@ Files in the former directory will take precedence over those in the\n\
 latter. The contents of the files @code{passwd} and @code{user_passwd}\n\
 (if present) must be kept secret.\n\
 \n\
+Notes: One of the two server files is named \"passwd\". It contains one\n\
+line per user -- the line starts with the username and a \":\". If this\n\
+file pre-exists, the new line will be appended to it. You may have to\n\
+manually merge this file with a potentially pre-existing file at the\n\
+server machine, e.g. if you want to add a new user there. Also, you\n\
+have to manually assure that in the version of this file at the server\n\
+machine each username appears only once (i.e. old lines for the same\n\
+user may have to be deleted). The username is arbitrary.\n\
+\n\
 This function zeroizes sensitive data before releasing its memory. Due\n\
 to usage of external libraries, however, it still can't be excluded\n\
 that sensitive data is still on the swap device after application\n\
@@ -165,7 +175,7 @@ shutdown.\n\
 
   // get password; not supposed to be free'd again, don't do it
   sensitive_string user_password
-    (getpass ("password (default is random password): "));
+    (gnulib_getpass ("password (default is random password): "));
   if (! user_password.good ())
     {
       error ("%s: could not read password", fname.c_str ());
@@ -242,7 +252,7 @@ shutdown.\n\
     }
 
   // password not needed anymore, zero and free it (but only zero it
-  // and not free it if it was made with 'getpass'
+  // and don't free it if it was made with 'getpass'
   user_password.free_val ();
 
   // write server passwd file or append if it exists
