@@ -1,11 +1,11 @@
 /*
 
 Copyright (C) 2002 Hayato Fujiwara <h_fujiwara@users.sourceforge.net>
-Copyright (C) 2010-2018 Olaf Till <i7tiol@t-online.de.de>
+Copyright (C) 2010-2020 Olaf Till <i7tiol@t-online.de.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
+the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -21,29 +21,15 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_ADD: autoload ("pconnect", "parallel_interface.oct");
 // PKG_DEL: autoload ("pconnect", "parallel_interface.oct", "remove");
 
-#include <octave/oct.h>
-
-#include "config.h"
-
-#include <octave/oct-env.h>
-
-#include <sys/socket.h>
-#include <errno.h>
-#include <netdb.h>
-#include <netinet/in.h> // reported necessary for FreeBSD-8
-
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include "parallel-gnutls.h"
 
 static
 int assert_file (std::string &path)
 {
-  OCTAVE__SYS__FILE_STAT stat (path);
+  // Windows headers expand stat(), so stat_ is used as object name
+  OCTAVE__SYS__FILE_STAT stat_ (path);
 
-  if (! stat.is_reg ())
+  if (! stat_.is_reg ())
     return -1;
   else
     return 0;
@@ -207,28 +193,17 @@ sclose (sockets);\n\
 
   // "canonicalize" host names and check for uniqueness
   Array<std::string> canhosts (dim_vector (nhosts, 1));
-  struct addrinfo *ai, hints;
-  memset ((void *) &hints, 0, sizeof (hints));
-  hints.ai_family = AF_INET;
-  hints.ai_socktype = SOCK_STREAM;
-  hints.ai_protocol = 0;
-  hints.ai_flags = AI_CANONNAME;
-  struct __aiguard
-  {
-    struct addrinfo *__ai;
-    __aiguard (struct addrinfo *__aai) : __ai (__aai) { }
-    ~__aiguard (void) { freeaddrinfo (__ai); }
-  };
   for (uint32_t i = 0; i < nhosts; i++)
     {
-      if (getaddrinfo (hosts(i).c_str (), NULL, &hints, &ai))
+      struct gnulib_guard __guard (&gnulib_freeaddrinfo);
+      canhosts(i) = std::string (gnulib_get_canonname (hosts(i).c_str ()));
+      if (canhosts(i).empty ())
         {
           error ("getaddrinfo returned an error");
           return retval;
         }
-      struct __aiguard __aig (ai);
-      canhosts(i) = std::string (ai->ai_canonname);
-      dcprintf ("orig: %s, canon: %s\n", hosts(i).c_str (), ai->ai_canonname);
+      dcprintf ("orig: %s, canon: %s\n",
+                hosts(i).c_str (), canhosts(i).c_str ());
     }
   Array<std::string> s_canhosts (canhosts.sort ());
   for (uint32_t i = 1; i < nhosts; i++)
@@ -437,37 +412,25 @@ sclose (sockets);\n\
   for (uint32_t i = 0; i < nhosts; i++)
     {
       dcprintf ("host number %i\n", i);
-      int sock = socket (PF_INET, SOCK_STREAM, 0);
+      int sock = gnulib_socket_pfinet_sockstream (0);
       if (sock == -1)
         {
           error ("socket error");
           return retval;
         }
-      struct __sockguard
-      {
-        __sockguard (int __isock) { __sock = __isock; }
-        ~__sockguard (void) { if (__sock > -1) close (__sock); }
-        void __release (void) { __sock = -1; }
-        int __sock;
-      } __sockg (sock);
+      struct gnulib_close_guard __sockg (sock);
 
-      struct addrinfo *ai, hints;
-      memset ((void *) &hints, 0, sizeof (hints));
-      hints.ai_family = AF_INET;
-      hints.ai_socktype = SOCK_STREAM;
-      hints.ai_protocol = 0;
-      hints.ai_flags = 0;
-      if (getaddrinfo (hosts(i).c_str (), "12502", &hints, &ai))
+      struct gnulib_guard __guard (&gnulib_freeaddrinfo);
+      if (gnulib_set_port (hosts(i).c_str (), "12502"))
         {
           error ("getaddrinfo returned an error");
           return retval;
         }
-      struct __aiguard __aig (ai);
 
       int not_connected = 1;
       for (int j = 0; j < N_CONNECT_RETRIES; j++)
         {
-          if((not_connected = connect (sock, ai->ai_addr, ai->ai_addrlen)) == 0)
+          if((not_connected = gnulib_connect (sock)) == 0)
             break;
           else if (errno != ECONNREFUSED && errno != EINTR)
             {
@@ -504,7 +467,7 @@ sclose (sockets);\n\
             conn->insert_cmd_stream
               (new octave_parallel_stream
                (new octave_parallel_socket_streambuf (sock, false)));
-          __sockg.__release ();
+          __sockg.release ();
           if (! conn->get_cmd_stream ()->good ())
             {
               error ("could not create command stream to %s",
@@ -533,7 +496,7 @@ sclose (sockets);\n\
               dcprintf ("hostname %i written (%s)\n", j, hosts(j).c_str ());
             }
 
-          std::string directory = octave_env::get_current_directory ();
+          std::string directory = OCTAVE__SYS__ENV::get_current_directory ();
 
           conn->get_cmd_stream ()->network_send_string (directory.c_str ());
           dcprintf ("directory written (%s)\n", directory.c_str ());
@@ -572,39 +535,27 @@ sclose (sockets);\n\
   for (uint32_t i = 0; i < nhosts; i++)
     {
       dcprintf ("host number %i\n", i);
-      struct addrinfo *ai = NULL, hints;
-      memset ((void *) &hints, 0, sizeof (hints));
-      hints.ai_family = AF_INET;
-      hints.ai_socktype = SOCK_STREAM;
-      hints.ai_protocol = 0;
-      hints.ai_flags = 0;
-      if (getaddrinfo (hosts(i).c_str (), "12501", &hints, &ai))
+      struct gnulib_guard __guard (&gnulib_freeaddrinfo);
+      if (gnulib_set_port (hosts(i).c_str (), "12501"))
         {
           error ("getaddrinfo returned an error");
           return retval;
         }
-      struct __aiguard __aig (ai);
 
       int not_connected = 1;
       for (int j = 0; j < N_CONNECT_RETRIES; j++)
         {
           dcprintf ("host %i, connect retry %i\n", i, j);
-          int sock = socket (PF_INET, SOCK_STREAM, 0);
+          int sock = gnulib_socket_pfinet_sockstream (0);
           if (sock == -1)
             {
               error ("socket error");
               return retval;
             }
-          struct __sockguard
-          {
-            __sockguard (int __isock) { __sock = __isock; }
-            ~__sockguard (void) { if (__sock > -1) close (__sock); }
-            void __release (void) { __sock = -1; }
-            int __sock;
-          } __sockg (sock);
+          struct gnulib_close_guard __sockg (sock);
 
           dcprintf ("%i, %i, trying to connect \n", i, j);
-          if (connect (sock, ai->ai_addr, ai->ai_addrlen) == 0)
+          if (gnulib_connect (sock) == 0)
             {
               dcprintf ("%i, %i, connect successful\n", i, j);
               octave_parallel_stream *data_stream;
@@ -619,7 +570,7 @@ sclose (sockets);\n\
                 conns->get_connections ()(i + 1)->insert_data_stream
                   (data_stream = new octave_parallel_stream
                    (new octave_parallel_socket_streambuf (sock, false)));
-              __sockg.__release ();
+              __sockg.release ();
               if (! data_stream->good ())
                 {
                   error ("%i, %i, could not create data stream to %s", i, j,
