@@ -1,10 +1,10 @@
 /*
 
-Copyright (C) 2015-2018 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015-2020 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
+the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -21,17 +21,15 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_DEL: autoload ("select_sockets", "parallel_interface.oct", "remove");
 
 #include <octave/oct.h>
-#include <octave/dirfns.h>
+
+#include "config.h"
+
+#ifdef HAVE_OCTAVE_DIRFNS_H
+  #include <octave/dirfns.h>
+#endif
 #include <octave/oct-map.h>
 #include <octave/oct-env.h>
 
-#include <stdio.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/select.h>
-#include <unistd.h>
-
 #include "parallel-gnutls.h"
 
 DEFUN_DLD (select_sockets, args, ,
@@ -80,7 +78,8 @@ received.\n\
     }
 
   double atimeout = args(1).double_value ();
-  int nfds = FD_SETSIZE;
+  int fd_setsize = gnulib_get_fd_setsize ();
+  int nfds = fd_setsize;
   bool err = false;
   if (args.length () == 3)
     {
@@ -98,7 +97,7 @@ received.\n\
 
       return retval;
     }
-  if (nfds > FD_SETSIZE)
+  if (nfds > fd_setsize)
     {
       error ("%s: 'nfds' exceeds systems maximum given by FD_SETSIZE",
              fname.c_str ());
@@ -129,20 +128,8 @@ received.\n\
           return retval;
         }
 
-  timeval tout;
-  timeval *timeout = &tout;
-  if (atimeout < 0)
-    timeout = NULL;
-  else
-    {
-      double ipart, fpart;
-      fpart = modf (atimeout, &ipart);
-      tout.tv_sec = lrint (ipart);
-      tout.tv_usec = lrint (fpart * 1000);
-    }
-
-  fd_set rfds, wfds, efds;
-  FD_ZERO (&rfds); FD_ZERO (&wfds); FD_ZERO (&efds);
+  gnulib_fd_zero_all ();
+  struct gnulib_guard __guard (&gnulib_fd_zero_all);
 
   inthandler_dont_restart_syscalls __inthandler_guard__;
 
@@ -161,7 +148,7 @@ received.\n\
           return retval;
         }
 
-      FD_SET (fid, &rfds);
+      gnulib_add_to_rfds (fid);
 
       if (! conns->is_server ())
         {
@@ -176,15 +163,17 @@ received.\n\
               return retval;
             }
 
-          FD_SET (fid, &rfds);
+          gnulib_add_to_rfds (fid);
         }
     }
 
   // don't restart interrupted select()
-  int n = select (nfds, &rfds, &wfds, &efds, timeout);
+  char *err_msg;
+  int n = gnulib_select (nfds, atimeout, &err_msg);
   if (n == -1)
     {
-      error ("%s: select system call returned an error", fname.c_str ());
+      error ("%s: gnulib_select returned an error: %s",
+             fname.c_str (), err_msg);
 
       return retval;
     }
@@ -198,10 +187,9 @@ received.\n\
       if (conns->get_connections ()(i)->is_pseudo_connection ())
         continue;
 
-      if (FD_ISSET
+      if (gnulib_fd_isset_rfds
           (conns->get_connections ()(i)->get_data_stream ()->
-           get_strbuf ()->get_fid (),
-           &rfds))
+           get_strbuf ()->get_fid ()))
         {
           *fvec = double (i + 1);
           fvec++;
@@ -209,10 +197,9 @@ received.\n\
 
       if (! conns->is_server ())
         {
-          if (FD_ISSET
+          if (gnulib_fd_isset_rfds
               (conns->get_connections ()(i)->get_cmd_stream ()->
-               get_strbuf ()->get_fid (),
-               &rfds))
+               get_strbuf ()->get_fid ()))
             {
               int err;
               if ((err = conns->get_connections ()(i)->poll_for_errors ()) < 0)
